# $Id: /mirror/perl/Data-Decode/trunk/lib/Data/Decode/Exception.pm 4834 2007-11-03T09:22:42.139028Z daisuke  $
#
# Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
# All rights reserved.

package Data::Decode::Exception;
use Exception::Class
    'Data::Decode::Exception::Deferred'
;

1;

__END__

=head1 NAME

Data::Decode::Exception - Exceptions

=cut